package com.abi.qms.platform.service.impl;

import com.abi.qms.platform.dao.entity.QrPackage;
import com.abi.qms.platform.dao.enums.QrPackageApplyStatusEnum;
import com.abi.qms.platform.dao.mapper.QrPackageMapper;
import com.abi.qms.platform.dao.vo.result.QrPackageVO;
import com.abi.qms.platform.dto.req.*;
import com.abi.qms.platform.dto.res.GetQrPackageDetailRes;
import com.abi.qms.platform.dto.res.ListQrPackageRes;
import com.abi.qms.platform.infrastructure.util.AssertUtil;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.QrPackageService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * 码包 Service业务层处理
 *
 * @author WeiganCai
 * @date 2021-04-28
 */
@Service
public class QrPackageServiceImpl implements QrPackageService {

	@Autowired
	private QrPackageMapper qrPackageMapper;

	/**
	 * 保存码包
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void saveQrPackage(SaveQrPackageReq req) {
		Long qrPackageId = req.getId();
		// 修改码包时的条件限制
		QrPackage qrPackage;
		if (qrPackageId != null) {
			qrPackage = qrPackageMapper.selectById(qrPackageId);
			AssertUtil.isNull(qrPackage, "码包不存在");
			// 只有待审核、审核拒绝状态下才可编辑
			Integer applyStatus = qrPackage.getApplyStatus();
			if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
				throw new BusinessException("码包不可编辑");
			}
		}

		// 码包名称不可重复
		QueryWrapper<QrPackage> qrPackageQw = new QueryWrapper<>();
		qrPackageQw.eq("name", req.getName());
		qrPackageQw.eq("is_delete", 0);
		if (qrPackageId != null) {
			qrPackageQw.ne("id", qrPackageId);
		}
		Integer count = qrPackageMapper.selectCount(qrPackageQw);
		if (!count.equals(0)) {
			throw new BusinessException("码包名称已存在");
		}

		// copy属性值
		qrPackage = PojoConverterUtils.copy(req, QrPackage.class);

		// 新增or修改
		Long userId = UserUtil.getUser().getId();
		qrPackage.setUpdateBy(userId);
		if (qrPackageId != null) {
			qrPackageMapper.updateById(qrPackage);
		} else {
			qrPackage.setCreateBy(userId);
			qrPackageMapper.insert(qrPackage);
		}

	}

	/**
	 * 码包分页查询
	 */
	@Override
	public ListQrPackageRes listQrPackage(ListQrPackageReq req) {
		// 分页查询
		IPage<QrPackageVO> iPage = qrPackageMapper.listQrPackage(PageUtil.createPage(req), req);
		List<QrPackageVO> qrPackageList = iPage.getRecords();

		// 封装出参、放入分页信息
		ListQrPackageRes res = new ListQrPackageRes();
		PageUtil.copyPageInfo(res, iPage);
		List<ListQrPackageRes.QrPackageBean> qrPackageBeanList = PojoConverterUtils.copyList(qrPackageList, ListQrPackageRes.QrPackageBean.class);
		res.setQrRepertoryBeanList(qrPackageBeanList);

		return res;
	}

	/**
	 * 码包查询详情
	 */
	@Override
	public GetQrPackageDetailRes getQrPackageDetail(GetQrPackageDetailReq req) {
		// 查询码包对象
		QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
		AssertUtil.isNull(qrPackage, "码包不存在");

		return PojoConverterUtils.copy(qrPackage, GetQrPackageDetailRes.class);
	}

	/**
	 * 删除码包
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void deleteQrPackage(DeleteQrPackageReq req) {
		QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
		AssertUtil.isNull(qrPackage, "码包不存在");
		// 只有待审核、审核拒绝状态下才可删除
		Integer applyStatus = qrPackage.getApplyStatus();
		if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
			throw new BusinessException("码包不可删除");
		}

		// 逻辑删除
		QrPackage update = new QrPackage().setId(req.getId()).setIsDelete(1);
		qrPackageMapper.updateById(update);
	}

	/**
	 * 审核通过码包
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void passQrPackage(PassQrPackageReq req) {
		QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
		AssertUtil.isNull(qrPackage, "码包不存在");

		// 只有待审核、审核拒绝状态下才可通过
		Integer applyStatus = qrPackage.getApplyStatus();
		if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
			throw new BusinessException("码包不可通过");
		}

		QrPackage update = new QrPackage().setId(req.getId()).setApplyStatus(QrPackageApplyStatusEnum.REVIEW_PASS.getCode());
		qrPackageMapper.updateById(update);

		// TODO 生成码
	}

	/**
	 * 审核拒绝码包
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public void refuseQrPackage(RefuseQrPackageReq req) {
		QrPackage qrPackage = qrPackageMapper.selectById(req.getId());
		AssertUtil.isNull(qrPackage, "码包不存在");

		// 只有待审核、审核拒绝状态下才可拒绝
		Integer applyStatus = qrPackage.getApplyStatus();
		if (QrPackageApplyStatusEnum.REVIEW_PASS.is(applyStatus)) {
			throw new BusinessException("码包不可拒绝");
		}

		QrPackage update = new QrPackage().setId(req.getId()).setApplyStatus(QrPackageApplyStatusEnum.REVIEW_REFUSE.getCode());
		qrPackageMapper.updateById(update);
	}
}
